@@ -1,11 +1,14 @@
 import logging
 from urllib.parse import urlparse
 
+from cryptojwt.jwt import JWT
+
 from oidcservice.client_auth import factory as ca_factory
 from oidcservice.exception import ResponseError
 from oidcservice.state_interface import StateInterface
 from oidcservice.util import get_http_body
 from oidcservice.util import get_http_url
+from oidcservice.util import JOSE_ENCODED
 from oidcservice.util import JSON_ENCODED
 from oidcservice.util import URL_ENCODED
 
@@ -91,9 +94,12 @@ class Service(StateInterface):
                         ar_args[prop] = self.conf['request_args'][prop]
                     except KeyError:
                         try:
-                            ar_args[prop] = self.default_request_args[prop]
+                            ar_args[prop] = self.service_context.register_args[prop]
                         except KeyError:
-                            pass
+                            try:
+                                ar_args[prop] = self.default_request_args[prop]
+                            except KeyError:
+                                pass
 
         return ar_args
 
@@ -177,11 +183,10 @@ class Service(StateInterface):
 
         # If 'state' appears among the keyword argument and is not
         # expected to appear in the request, remove it.
-        if 'state' not in self.msg_type.c_param:
-            try:
-                del kwargs['state']
-            except KeyError:
-                pass
+        if 'state' in self.msg_type.c_param and 'state' in kwargs:
+            # Don't overwrite something put there by the constructor
+            if 'state' not in request_args:
+                request_args['state'] = kwargs['state']
 
         # logger.debug("request_args: %s" % sanitize(request_args))
         _args = self.gather_request_args(**request_args)
@@ -278,12 +283,13 @@ class Service(StateInterface):
         """
         return self.default_authn_method
 
-    def get_request_parameters(self, request_body_type="", method="", authn_method='',
-                               request_args=None, http_args=None, **kwargs):
+    def get_request_parameters(self, request_body_type="", method="",
+                               authn_method='', request_args=None,
+                               http_args=None, **kwargs):
         """
         Builds the request message and constructs the HTTP headers.
 
-        This is the starting pont for a pipeline that will:
+        This is the starting point for a pipeline that will:
 
         - construct the request message
         - add/remove information to/from the request message in the way a
@@ -335,6 +341,8 @@ class Service(StateInterface):
             # How should it be serialized
             if request_body_type == 'urlencoded':
                 content_type = URL_ENCODED
+            elif request_body_type in ['jws','jwe', 'jose']:
+                content_type = JOSE_ENCODED
             else:  # request_body_type == 'json'
                 content_type = JSON_ENCODED
 
@@ -416,11 +424,31 @@ class Service(StateInterface):
 
         logger.debug('response format: {}'.format(sformat))
 
+        if sformat in ['jose','jws','jwe']:
+            resp = self.post_parse_response(info, state=state)
+
+            if not resp:
+                logger.error('Missing or faulty response')
+                raise ResponseError("Missing or faulty response")
+
+            return resp
+
         # If format is urlencoded 'info' may be a URL
         # in which case I have to get at the query/fragment part
         if sformat == "urlencoded":
             info = self.get_urlinfo(info)
 
+        if sformat == 'jwt':
+            args = {'allowed_sign_algs':
+                        self.service_context.get_sign_alg(self.service_name)}
+            enc_algs = self.service_context.get_enc_alg_enc(self.service_name)
+            args['allowed_enc_algs'] = enc_algs['alg']
+            args['allowed_enc_encs'] = enc_algs['enc']
+            _jwt = JWT(key_jar=self.service_context.keyjar, **args)
+            _jwt.iss = self.service_context.client_id
+            info = _jwt.unpack(info)
+            sformat = "dict"
+
         logger.debug('response_cls: {}'.format(self.response_cls.__name__))
         try:
             resp = self.response_cls().deserialize(
@@ -434,7 +462,7 @@ class Service(StateInterface):
                 try:
                     resp = self.response_cls().deserialize(
                         info, 'jwt', iss=self.service_context.issuer, **kwargs)
-                except Exception as err:
+                except Exception as err2:
                     pass
 
             if resp is None:
@@ -483,8 +511,8 @@ class Service(StateInterface):
             return default
 
 
-def build_services(service_definitions, service_factory, service_context,
-                   state_db, client_authn_factory=None):
+def build_services(service_definitions, service_factory, module_dirs,
+                   service_context, state_db, client_authn_factory=None):
     """
     This function will build a number of :py:class:`oidcservice.service.Service`
     instances based on the service definitions provided.
@@ -492,6 +520,10 @@ def build_services(service_definitions, service_factory, service_context,
     :param service_definitions: A dictionary of service definitions. The keys
         are the names of the subclasses. The values are configurations.
     :param service_factory: A factory that can initiate a service class
+    :param module_dirs: The directories in which to search for service modules.
+        Directory names are expected to be relative to the oidcservice package.
+        e.g. ['oidc', 'oauth2]. The directories will be search in order until
+        a matching module is found.
     :param service_context: A reference to the service context, this is the same
         for all service instances.
     :param state_db: A reference to the state database. Shared by all the
@@ -503,10 +535,15 @@ def build_services(service_definitions, service_factory, service_context,
     """
     service = {}
     for service_name, service_configuration in service_definitions.items():
-        _srv = service_factory(service_name, service_context=service_context,
+        _srv = service_factory(service_name,
+                               module_dirs,
+                               service_context=service_context,
                                state_db=state_db,
                                client_authn_factory=client_authn_factory,
                                conf=service_configuration)
-        service[_srv.service_name] = _srv
+        try:
+            service[_srv.service_name] = _srv
+        except AttributeError:
+            raise ValueError("Could not load '{}'".format(service_name))
 
     return service
